@@ -35,6 +35,11 @@ Entry point classes of Flink DataStream API:
       Represent two connected streams of (possibly) different data types. Connected
       streams are useful for cases where operations on one stream directly affect the operations on
       the other stream, usually via shared state between the streams.
+    - :class:`BroadcastStream`:
+      Represent a stream with :class:`state.BroadcastState` (s).
+    - :class:`BroadcastConnectedStream`:
+      Represents the result of connecting a keyed or non-keyed stream, with a
+      :class:`BroadcastStream` with :class:`state.BroadcastState` (s)
 
 Functions used to transform a :class:`DataStream` into another :class:`DataStream`:
 
@@ -57,8 +62,11 @@ Functions used to transform a :class:`DataStream` into another :class:`DataStrea
     - :class:`KeyedProcessFunction`:
       Similar to :class:`ProcessFunction`, except that it was applied to a :class:`KeyedStream` and
       could register event-time and processing-time timers.
+    - :class:`CoProcessFunction`:
+      Similar to :class:`CoFlatMapFunction`, except that it could access the current timestamp and
+      watermark in :class:`CoProcessFunction`.
     - :class:`KeyedCoProcessFunction`:
-      Similar to :class:`CoFlatMapFunction`, except that it was applied to a keyed
+      Similar to :class:`CoProcessFunction`, except that it was applied to a keyed
       :class:`ConnectedStreams` and could register event-time and processing-time timers.
     - :class:`WindowFunction`:
       Base interface for functions that are evaluated over keyed (grouped) windows.
@@ -67,6 +75,13 @@ Functions used to transform a :class:`DataStream` into another :class:`DataStrea
       information such as the current timestamp, the watermark, etc.
     - :class:`AggregateFunction`:
       Base class for a user-defined aggregate function.
+    - :class:`BroadcastProcessFunction`:
+      A function to be applied to a :class:`BroadcastConnectedStream` that connects
+      :class:`BroadcastStream`, i.e. a stream with broadcast state, with a non-keyed
+      :class:`DataStream`.
+    - :class:`KeyedBroadcastProcessFunction`:
+      A function to be applied to a :class:`BroadcastConnectedStream` that connects
+      :class:`BroadcastStream`, i.e. a stream with broadcast state, with a :class:`KeyedStream`.
     - :class:`RuntimeContext`:
       Contains information about the context in which functions are executed. Each
       parallel instance of the function will have a context through which it can access static
@@ -80,6 +95,8 @@ Classes to define window:
       A grouping of elements according to a time interval from start (inclusive) to end (exclusive).
     - :class:`CountWindow`:
       A grouping of elements according to element count from start (inclusive) to end (exclusive).
+    - :class:`GlobalWindow`:
+      The window into which all data is placed.
     - :class:`WindowAssigner`:
       Assigns zero or more :class:`Window` to an element.
     - :class:`MergingWindowAssigner`:
@@ -101,20 +118,23 @@ Classes to define the behavior of checkpoint and state backend:
       Base class of the state backends which define how the state of a streaming application is
       stored locally within the cluster. Different state backends store their state in different
       fashions, and use different data structures to hold the state of a running application.
-    - :class:`MemoryStateBackend`:
-      holds the working state in the memory (JVM heap) of the TaskManagers.
-      The state backend checkpoints state directly to the JobManager's memory (hence the backend's
-      name), but the checkpoints will be persisted to a file system for high-availability setups and
-      savepoints. The MemoryStateBackend is consequently a FileSystem-based backend that can work
-      without a file system dependency in simple setups.
-    - :class:`RocksDBStateBackend`:
-      Stores its state in ``RocksDB``. This state backend can store very large state that
-      exceeds memory and spills to disk.
-    - :class:`FsStateBackend`:
-      Holds the working state in the memory (JVM heap) of the TaskManagers.
-      The state backend checkpoints state as files to a file system (hence the backend's name).
+    - :class:`HashMapStateBackend`:
+      Holds the working state in the memory (JVM heap) of the TaskManagers and
+      checkpoints based on the configured :class:`CheckpointStorage`.
+    - :class:`EmbeddedRocksDBStateBackend`:
+      Stores its state in an embedded `RocksDB` instance. This state backend can store very large
+      state that exceeds memory and spills to local disk.
     - :class:`CustomStateBackend`:
       A wrapper of customized java state backend.
+    - :class:`JobManagerCheckpointStorage`:
+      Checkpoints state directly to the JobManager's memory (hence the name), but savepoints will
+      be persisted to a file system.
+    - :class:`FileSystemCheckpointStorage`:
+      Checkpoints state as files to a file system. Each checkpoint individually will store all its
+      files in a subdirectory that includes the checkpoint number, such as
+      `hdfs://namenode:port/flink-checkpoints/chk-17/`.
+    - :class:`CustomCheckpointStorage`:
+      A wrapper of customized java checkpoint storage.
 
 Classes for state operations:
 
@@ -133,28 +153,93 @@ Classes for state operations:
     - :class:`state.AggregatingState`:
       Interface for aggregating state, based on an :class:`AggregateFunction`. Elements that are
       added to this type of state will be eagerly pre-aggregated using a given AggregateFunction.
+    - :class:`state.BroadcastState`:
+      A type of state that can be created to store the state of a :class:`BroadcastStream`. This
+      state assumes that the same elements are sent to all instances of an operator.
+    - :class:`state.ReadOnlyBroadcastState`:
+      A read-only view of the :class:`state.BroadcastState`.
+    - :class:`state.StateTtlConfig`:
+      Configuration of state TTL logic.
 
 Classes to define source & sink:
 
-    - :class:`connectors.FlinkKafkaConsumer`:
+    - :class:`connectors.elasticsearch.ElasticsearchSink`:
+      A sink for publishing data into Elasticsearch 6 or Elasticsearch 7.
+    - :class:`connectors.kafka.FlinkKafkaConsumer`:
       A streaming data source that pulls a parallel data stream from Apache Kafka.
-    - :class:`connectors.FlinkKafkaProducer`:
+    - :class:`connectors.kafka.FlinkKafkaProducer`:
       A streaming data sink to produce data into a Kafka topic.
-    - :class:`connectors.FileSource`:
+    - :class:`connectors.kafka.KafkaSource`:
+      The new API to read data in parallel from Apache Kafka.
+    - :class:`connectors.kafka.KafkaSink`:
+      The new API to write data into to Apache Kafka topics.
+    - :class:`connectors.file_system.FileSource`:
       A unified data source that reads files - both in batch and in streaming mode.
       This source supports all (distributed) file systems and object stores that can be accessed via
       the Flink's FileSystem class.
-    - :class:`connectors.FileSink`:
+    - :class:`connectors.file_system.FileSink`:
       A unified sink that emits its input elements to FileSystem files within buckets. This
       sink achieves exactly-once semantics for both BATCH and STREAMING.
-    - :class:`connectors.NumberSequenceSource`:
-      A data source that produces a sequence of numbers (longs). This source is useful for testing
-      and for cases that just need a stream of N events of any kind.
-    - :class:`connectors.JdbcSink`:
-      A data sink to produce data into an external storage using JDBC.
-    - :class:`connectors.StreamingFileSink`:
+    - :class:`connectors.file_system.StreamingFileSink`:
       Sink that emits its input elements to files within buckets. This is integrated with the
       checkpointing mechanism to provide exactly once semantics.
+    - :class:`connectors.number_seq.NumberSequenceSource`:
+      A data source that produces a sequence of numbers (longs). This source is useful for testing
+      and for cases that just need a stream of N events of any kind.
+    - :class:`connectors.jdbc.JdbcSink`:
+      A data sink to produce data into an external storage using JDBC.
+    - :class:`connectors.pulsar.PulsarSource`:
+      A streaming data source that pulls a parallel data stream from Pulsar.
+    - :class:`connectors.pulsar.PulsarSink`:
+      A streaming data sink to produce data into Pulsar.
+    - :class:`connectors.rabbitmq.RMQSource`:
+      A streaming data source that pulls a parallel data stream from RabbitMQ.
+    - :class:`connectors.rabbitmq.RMQSink`:
+      A Sink for publishing data into RabbitMQ.
+    - :class:`connectors.cassandra.CassandraSink`:
+      A Sink for publishing data into Cassandra.
+    - :class:`connectors.kinesis.FlinkKinesisConsumer`:
+      A streaming data source that pulls a parallel data stream from Kinesis.
+    - :class:`connectors.kinesis.KinesisStreamsSink`:
+      A Kinesis Data Streams (KDS) Sink that performs async requests against a destination stream
+      using the buffering protocol.
+    - :class:`connectors.kinesis.KinesisFirehoseSink`:
+      A Kinesis Data Firehose (KDF) Sink that performs async requests against a destination delivery
+      stream using the buffering protocol.
+    - :class:`connectors.hybrid_source.HybridSource`:
+      A Hybrid source that switches underlying sources based on configured source chain.
+
+
+Classes to define formats used together with source & sink:
+
+    - :class:`formats.csv.CsvReaderFormat`:
+      A :class:`~connectors.file_system.StreamFormat` to read CSV files into Row data.
+    - :class:`formats.csv.CsvBulkWriter`:
+      Creates :class:`~pyflink.common.serialization.BulkWriterFactory` to write Row data into CSV
+      files.
+    - :class:`formats.avro.GenericRecordAvroTypeInfo`:
+      A :class:`~pyflink.common.typeinfo.TypeInformation` to indicate vanilla Python records will be
+      translated to GenericRecordAvroTypeInfo on the Java side.
+    - :class:`formats.avro.AvroInputFormat`:
+      An InputFormat to read avro files in a streaming fashion.
+    - :class:`formats.avro.AvroWriters`:
+      A class to provide :class:`~pyflink.common.serialization.BulkWriterFactory` to write vanilla
+      Python objects into avro files in a batch fashion.
+    - :class:`formats.parquet.ParquetColumnarRowInputFormat`:
+      A :class:`~connectors.file_system.BulkFormat` to read columnar parquet files into Row data in
+      a batch-processing fashion.
+    - :class:`formats.parquet.ParquetBulkWriters`:
+      Convenient builder to create a :class:`~pyflink.common.serialization.BulkWriterFactory` that
+      writes Rows with a defined RowType into Parquet files in a batch fashion.
+    - :class:`formats.parquet.AvroParquetReaders`:
+      A convenience builder to create reader format that reads individual Avro records from a
+      Parquet stream. Only GenericRecord is supported in PyFlink.
+    - :class:`formats.parquet.AvroParquetWriters`:
+      Convenience builder to create ParquetWriterFactory instances for Avro types. Only
+      GenericRecord is supported in PyFlink.
+    - :class:`formats.orc.OrcBulkWriters`:
+      Convenient builder to create a :class:`~pyflink.common.serialization.BulkWriterFactory` that
+      writes Row records with a defined :class:`RowType` into Orc files.
 
 Other important classes:
 
@@ -171,28 +256,37 @@ Other important classes:
       Interface for implementing user defined sink functionality.
     - :class:`SourceFunction`:
       Interface for implementing user defined source functionality.
+    - :class:`OutputTag`:
+      Tag with a name and type for identifying side output of an operator
 """
 from pyflink.datastream.checkpoint_config import CheckpointConfig, ExternalizedCheckpointCleanup
 from pyflink.datastream.checkpointing_mode import CheckpointingMode
 from pyflink.datastream.data_stream import DataStream, KeyedStream, WindowedStream, \
-    ConnectedStreams, DataStreamSink
+    ConnectedStreams, DataStreamSink, BroadcastStream, BroadcastConnectedStream
 from pyflink.datastream.execution_mode import RuntimeExecutionMode
 from pyflink.datastream.functions import (MapFunction, CoMapFunction, FlatMapFunction,
                                           CoFlatMapFunction, ReduceFunction, RuntimeContext,
                                           KeySelector, FilterFunction, Partitioner, SourceFunction,
-                                          SinkFunction, KeyedProcessFunction,
+                                          SinkFunction, CoProcessFunction, KeyedProcessFunction,
                                           KeyedCoProcessFunction, AggregateFunction, WindowFunction,
-                                          ProcessWindowFunction)
+                                          ProcessWindowFunction, BroadcastProcessFunction,
+                                          KeyedBroadcastProcessFunction)
+from pyflink.datastream.slot_sharing_group import SlotSharingGroup, MemorySize
 from pyflink.datastream.state_backend import (StateBackend, MemoryStateBackend, FsStateBackend,
                                               RocksDBStateBackend, CustomStateBackend,
-                                              PredefinedOptions)
+                                              PredefinedOptions, HashMapStateBackend,
+                                              EmbeddedRocksDBStateBackend)
+from pyflink.datastream.checkpoint_storage import (CheckpointStorage, JobManagerCheckpointStorage,
+                                                   FileSystemCheckpointStorage,
+                                                   CustomCheckpointStorage)
 from pyflink.datastream.stream_execution_environment import StreamExecutionEnvironment
 from pyflink.datastream.time_characteristic import TimeCharacteristic
 from pyflink.datastream.time_domain import TimeDomain
 from pyflink.datastream.functions import ProcessFunction
 from pyflink.datastream.timerservice import TimerService
 from pyflink.datastream.window import Window, TimeWindow, CountWindow, WindowAssigner, \
-    MergingWindowAssigner, TriggerResult, Trigger
+    MergingWindowAssigner, TriggerResult, Trigger, GlobalWindow
+from pyflink.datastream.output_tag import OutputTag
 
 __all__ = [
     'StreamExecutionEnvironment',
@@ -200,6 +294,8 @@ __all__ = [
     'KeyedStream',
     'WindowedStream',
     'ConnectedStreams',
+    'BroadcastStream',
+    'BroadcastConnectedStream',
     'DataStreamSink',
     'MapFunction',
     'CoMapFunction',
@@ -209,25 +305,35 @@ __all__ = [
     'FilterFunction',
     'ProcessFunction',
     'KeyedProcessFunction',
+    'CoProcessFunction',
     'KeyedCoProcessFunction',
     'WindowFunction',
     'ProcessWindowFunction',
     'AggregateFunction',
+    'BroadcastProcessFunction',
+    'KeyedBroadcastProcessFunction',
     'RuntimeContext',
     'TimerService',
     'CheckpointingMode',
     'CheckpointConfig',
     'ExternalizedCheckpointCleanup',
     'StateBackend',
+    'HashMapStateBackend',
+    'EmbeddedRocksDBStateBackend',
     'CustomStateBackend',
     'MemoryStateBackend',
     'RocksDBStateBackend',
     'FsStateBackend',
     'PredefinedOptions',
+    'CheckpointStorage',
+    'JobManagerCheckpointStorage',
+    'FileSystemCheckpointStorage',
+    'CustomCheckpointStorage',
     'RuntimeExecutionMode',
     'Window',
     'TimeWindow',
     'CountWindow',
+    'GlobalWindow',
     'WindowAssigner',
     'MergingWindowAssigner',
     'TriggerResult',
@@ -237,5 +343,8 @@ __all__ = [
     'KeySelector',
     'Partitioner',
     'SourceFunction',
-    'SinkFunction'
+    'SinkFunction',
+    'SlotSharingGroup',
+    'MemorySize',
+    'OutputTag'
 ]
